let state = 3; // Start with state 3(intro)
let earth;
let sun;
let sound1;
let sound2;
let myFont;

//Preload soounds and font
function preload() {
	sound1 = loadSound('peace.wav');
	sound2 = loadSound('sad.mp4');
	myFont = loadFont('font1.ttf');
  }

function setup() {
	createCanvas(windowWidth, windowHeight, WEBGL); 	 // 3D canvas
	earth = new Planet(150, 0, 0, 0, 0, 0, 255, 1);		 // An object of the Planet class
	sun = new Planet(360, -800, 0, -60, 255, 180, 0, 4); // An object of the Planet class
	angleMode(DEGREES);
	strokeWeight(3);
	noFill();
	frameRate(24);
}   

function draw() {
	background(0);

	if (state == 1) {
	    earth.movingSphere();
		sun.movingSphere();
	}

	if (state == 2) {
		earth.movingShakingSphere();
		sun.movingShakingSphere();
	}

	if (state == 3) {
		waves();
	}
}

// Adjust canvas sixe when the browser window is smaller
function windowResized() {
	resizeCanvas(windowWidth, windowHeight);
  }
  

function keyPressed() {

	if (keyCode === 49) {			// Key 1
		state = 1;
		sound2.stop();
		sound1.play();
	    sound1.loop();
	} else if (keyCode === 50) {	//Key 2
		sound1.stop();
		sound2.play();
	    sound2.loop();
		state = 2;
	} else if (keyCode === 51) {	//Key 3
		sound1.stop();
		sound2.stop();
		state = 3;

	}
}


//Class planet for creating spheres which consist of lots of small spheres
class Planet {

	constructor (r, x2, y2, z2, red, green, blue, smR) {
		this.r = r;			// Radius of the planet
		this.x2 = x2;		// Position X
		this.y2 = y2;       // Position Y
		this.z2 = z2;		// Position Z
		this.red = red;
		this.green = green;
		this.blue = blue;
		this.smR = smR;		// Radius of small spheres 
	}

	movingSphere() {

		push();
		translate(this.x2, this.y2, this.z2);
		
		rotateZ(90);
	    rotateX(frameCount * 0.5);

		// Reference for line 94-105: timecode - 3:44
		//Kazuki Umeda (2021) What We can Create w/ p5js & Spherical Coordinates YouTube. 15 July 2021 [online]. Available from: https://www.youtube.com/watch?v=SGHWZz5Mrsw [Accessed 3 April 2025].
		for (let phi = 0; phi < 180; phi += 15) {
			for (let theta = 0; theta < 360; theta += 15) {

				if (this === earth) {
					this.changingColour(phi, theta);
				} else {
					stroke(this.red, this.green, this.blue);
				}
	
				let x = this.r * cos(phi);
				let y = this.r * sin(phi) * sin(theta);
				let z = this.r * sin(phi) * cos(theta);
	
				push();
				translate(x ,y, z);
				sphere(this.smR);
				pop();
	
			}		
		}
		pop();

	}

	changingColour(phi, theta) {

		stroke(this.red, this.green, this.blue);

		// Array with all coordinates that should be filled in green colour 
		// coordinates[i][0] = start phi
		// coordinates[i][1] = start theta
		// coordinates[i][2] = last phi
		// coordinates[i][3] = last theta
		let coordinates = [
			[75, 210, -1, -1], 
			[90, 195, -1, -1],
			[30, 195, -1, -1],
			[30, 165, -1, -1],
			[150, 15, -1 , -1],
			[0, 165, 105, -1],
			[0, 180 , 90, -1],
			[30, 150, 90, -1],
			[60, 195, 75, -1],
			[120, 150, -1, 180],
			[135, 135, -1, 210],
			[150, 150, -1, 210],
			[165, 150, -1, 180],
			[135, 0, -1, 15],
			[135, 300, -1, 360],
			[120, 0, -1, 45],
			[120, 285, -1, 360], 
			[120, 60, -1, -1],
			[105, 0, -1, 45], 
			[105, 300, -1, 360],
			[90, 0, -1, 30],
			[90, 315, -1, 330],
			[75, 15, -1, 45],
			[75, 330, -1, -1],
			[60, 15, -1, 45],
			[60, 300, -1, -1],
			[45, 285, -1, 330],
			[45, 15, -1, 45],
			[30, 30, -1, -1],
			[30, 285, -1, 330]
		];

		// Logic for highlighting spheres in green colour
		for (let i = 0; i < coordinates.length; i++) {

			// If multiple spheres in the same column
			if (coordinates[i][2] != - 1) {
				for (let first = coordinates[i][0]; first <= coordinates[i][2]; first += 15) {
					if (phi == first && theta == coordinates[i][1]) {
						stroke(0, 255, 0);
					}
				}
			// If multiple spheres in the same row
			}else if (coordinates[i][3] != -1) {
				for (let first = coordinates[i][1]; first <= coordinates[i][3]; first += 15) {
					if (phi == coordinates[i][0] && theta == first) {
						stroke(0, 255, 0);
					}
				}
			}else if (phi == coordinates[i][0] && theta == coordinates[i][1]) {
				stroke(0, 255, 0);
			}
		}		

	}

	movingShakingSphere() {

		push();
		translate(this.x2, this.y2, this.z2);
		
		rotateZ(90);
	    rotateX(frameCount * 0.5);

		for (let phi = 0; phi < 180; phi += 15) {
			for (let theta = 0; theta < 360; theta += 15) {

				if (this === earth) {
					this.changingColour(phi, theta);
				} else {
					stroke(this.red, this.green, this.blue);
				}

				let noiseVal = noise(random(0.3, 1.1));

				// Imitating a shaking sphere by randomising the position of each small sphere
				let x = this.r * cos(phi) * noiseVal;
				let y = this.r * sin(phi) * sin(theta) * noiseVal;
				let z = this.r * sin(phi) * cos(theta) * noiseVal;;
	
				push();
				translate(x ,y, z);
				sphere(this.smR);
				pop();
	
			}		
		}
		pop();
	}
}

//Function for creating introduction state with description of the project
function waves() {
	background('black');
	push();
	translate(-width / 2, -height / 2, 0);

	// Refernce line 226 - 233: ‘noise’ (2024) P5js.org. 2024 [online]. Available from: https://p5js.org/reference/p5/noise/.
	let noiseLevel = 300;
	let noiseScale = 0.004;
	stroke(255, 0, 0);
	for (let x = 0; x < windowWidth; x += 5) {
		let nx = noiseScale * x;
		let nt = noiseScale * frameCount;
		let y = noiseLevel * noise(nx, nt);
		line(x, 0, x, y);
	}
	pop();

	// Display intro text
	push();
	textFont(myFont);
	textSize(32);
	fill(255);
	noStroke();
	textAlign(CENTER, CENTER);
	text("'From calm to chaos'\nPress '1' or '2' to see other states, and '3' to return back to this page", 0, 0);
	textSize(16);
	text("This project represents my personal feelings about situation in my home country - Ukraine.\nUsing part of Solar System, I aimed to convey my emotions before war and during the war, as thouhg even Earth has changed behaviour at this time.\nPress '1' to see my state before February 2022, and '2' to see my state after.", 0, 80);
	pop();
}
